//
//  AgentModel.h
//  Creatures
//
//  Created by Lenka on 23/12/2009.
//  Copyright 2009 Lenka Pitonakova. All rights reserved.
//

/*   DynObjModel
 
 - base class for all dynamic object's models
 - a subclass should implement the following:
	1. Custom init function (eg. - (id) initMyModelWithController: (DynObjController*)control_;)
	2. In the init function AFTER INITIALISING CLASS VARIABLES call [super initModelWithController: encodingKey:] The encoding key corresponds to a name under which a model is encoded and should be unique for each class.
	3. Encoding / decoding methods:
		A) - (void) encodeWithCoder: (NSCoder*) encoder
		B) - (id) initWithCoder: (NSCoder *) decoder
		C) - (void) SetSelfWith: (DynObjModel*)decodedObj_
	4. Make all encoded variables properties of the class
 
	Further details about these functios in DynObjModel.m
 
*/

#import <Foundation/Foundation.h>
#import "Constants.h"

#define k1 @"var1"
#define k2 @"var2"
#define k3 @"var3"
#define k4 @"var4"
#define k5 @"var5"
#define k6 @"var6"
#define k7 @"var7"
#define k8 @"var8"
#define k9 @"var9"
#define k10 @"var10"

@class DynObjController;

@interface DynObjModel : NSObject <NSCoding> {
	DynObjController* m_pController;
	NSString * m_sEncodingKey;
}

@property(nonatomic, retain) DynObjController *m_pController;

- (id) initModelWithController:(DynObjController*)controller_ encodingKey:(NSString*)key_;
- (void) SetSelfWith: (DynObjModel*)decodedObj_;

@end
