//
//  AgentController.m
//  Creatures
//
//  Created by Lenka on 21/12/2009.
//  Copyright 2009 Lenka Pitonakova. All rights reserved.
//


/*  REFERENCES
 [1] http://stackoverflow.com/questions/1230289/override-classes-variable-or-at-least-variable-type-in-objective-c-cocoa (accessed 10 Jan 2010)
*/
#import "UnitController.h"

#import "UnitMotor.h"
#import "World.h"
#import "DynObjMemory.h"
#import "UnitView.h"
#import "UnitDNA.h"

@implementation UnitController

/* CONSTRUCTOR 1 
 - used from World class when starting the program
*/
- (id) initUnit: (Byte)id_ pos:(CGPoint)pos_ rot:(CGFloat)rot_ world:(World*)world_{
	[super initDynObj:id_ type:OBJ_UNIT pos:pos_ rot:rot_ world:world_];
	m_pDNA = [[UnitDNA alloc] initDNAWithController:self];
	[self SetupSelf];
	return self;
}

/* CONSTRUCTOR 2
 - used from World class when a unit is added after breeding using a specific DNA string
*/
- (id) initUnit: (Byte)id_ pos:(CGPoint)pos_ rot:(CGFloat)rot_ world:(World*)world_ DNA:(NSString*)dna_{
	[super initDynObj:id_ type:OBJ_UNIT pos:pos_ rot:rot_ world:world_];
	m_pDNA = [[UnitDNA alloc] initDNAWithController:self];
	m_pDNA.m_sDNAString = dna_;
	[self SetupSelf];
	return self;
}

- (void) SetupSelf {
	//------ change texture:
	m_iTextureId = TEX_UNITS_BODIES;
	m_pMotor = [[UnitMotor alloc] initMotorWithController:self];
	m_pView = [[UnitView alloc] initViewWithController:self];	
	//----------- determine slice of the units texture:
	m_ptUcoordins = CGPointMake(0,0.5);
	Byte bodyCol = [m_pDNA GetBodyColor];
	switch (bodyCol) {
		case COL_PURPLE:
			m_ptVcoordins = CGPointMake(0,0.25);
			break;
		case COL_RED:
			m_ptVcoordins = CGPointMake(0.25,0.5);
			break;
		case COL_BLUE:
			m_ptVcoordins = CGPointMake(0.5,0.75);
			break;
		case COL_GREEN:
			m_ptVcoordins = CGPointMake(0.75,1);
			break;
		default:
			break;
	}
}

- (void) dealloc {
	[m_pMotor release];
	[m_pDNA dealloc];
	[m_pDNA release];
	[super dealloc];
}
- (void) Update {
	[super Update];
	[m_pMemory Update];
	//--------------------- decide what to do based on the current task:
	if ([m_pMemory GetCurTaskName] == UORDER_MOVE ) {
		[m_pMotor MoveTowards:[m_pMemory GetCurTaskLocation]];
	} else if ([m_pMemory GetCurTaskName] == UORDER_BREED) {
		CGPoint pos = [m_pMemory GetCurTaskTargetObj].m_ptPos;
		[m_pMotor MoveTowards:pos];
	}
}

- (void) DoRendering {
	//----------- call UnitView's overriden RenderAt function  [REF 1]:
	[(UnitView*)m_pView  RenderAt:m_ptPos rot:m_fRot txtrId:m_iTextureId uCoordins:m_ptUcoordins vCoordins:m_ptVcoordins eyes:[m_pDNA GetEyesColor]];
}

- (NSString*) GetDNA { 	return m_pDNA.m_sDNAString; }


#pragma mark ================================ USER INTERACTION


- (void) FinishedTask:(Byte)task_  {
	switch (task_) {
		case UORDER_BREED: {
			NSString* newDNA = [m_pDNA GenerateChildStringWith:[(UnitController*)[m_pMemory GetCurTaskTargetObj] GetDNA]];
			[m_pWorld AddUnitAt:CGPointMake(m_ptPos.x, m_ptPos.y) rot:0 withDNA:newDNA];
		}
			break;
	}
	[super FinishedTask: task_];
}

#pragma mark ================================= TOUCHES
/*- (void) HandleTouchesBeginAt:(CGPoint)pos_ {
	
	[m_pView HandleTouchesBeginAt:pos_ selfPos:m_pMotor.m_ptPos selfSize:m_pMotor.m_ptSize];
	
}

- (void) HandleTouchesEndedAt:(CGPoint)loc_ {
	[m_pView HandleTouchesEndedAt:loc_ selfPos:m_pMotor.m_ptPos selfSize:m_pMotor.m_ptSize];
	
}*/





@end
