//
//  AgentMotor.m
//  Creatures
//
//  Created by Lenka on 21/12/2009.
//  Copyright 2009 Lenka Pitonakova. All rights reserved.
//

#import "UnitMotor.h"
#import "Vector.h"
#import "UnitController.h"


@implementation UnitMotor


@synthesize m_sMotors;

/* 
 main init method, called each time application starts from the AgentController class
*/
- (id) initMotorWithController:(UnitController*)control_ {
	m_sMotors.steeringL = m_sMotors.steeringR = 0;
	m_sMotors.thrustBack = m_sMotors.thrustFront = 0;
	//---------------- the super class method which handles decoding and subscribes to applicationWillTermina notification:
	[super initModelWithController:control_ encodingKey:@"dataMotor"];
	return self;
}


- (void) MoveTowards: (CGPoint)loc_ {
	CGPoint localVec = ConvertPointToLocalCoordins (m_pController.m_fRot,SubtractVectors(loc_, m_pController.m_ptPos));
	//------------ adjust speed according to the distance of the target, minimal speed is 3:
	m_sMotors.thrustBack = 3 + fabs(localVec.y / 50);
	//------------ normalize the local vector to get only direction:
	localVec = NormalizeVector(localVec);
	//----------- use vector's x component to turn:
	[self TurnBy:localVec.x*5];
	//------------------ test if user clicked along the local horizontal axis:
	if (localVec.x < 0.1 && localVec.x > -0.1) {
		//----------- test if click was behind:
		if (localVec.y > 0) {
			//------- make a big turn:
			[self TurnBy:90];
		}
	}
	[self GoForward];
	
}

#pragma mark =================================== BASIC

- (void) GoForward {
	m_pController.m_ptPos = AddVectors(m_pController.m_ptPos, CGPointMake(
											cos(m_pController.m_fRot*PI/180)*m_sMotors.thrustBack,
											sin(m_pController.m_fRot*PI/180)*m_sMotors.thrustBack));

}

- (void) TurnBy:(CGFloat) angle_ {
	m_pController.m_fRot += angle_;
}


#pragma mark ======================================= NSCoding

/* no archiving neccessary, UnitMotor does not have any instance variables which need to persist */
@end
