//
//  World.m
//  Creatures
//
//  Created by Lenka on 07/12/2009.
//  Copyright 2009 Lenka Pitonakova. All rights reserved.
//

/* REFERENCES
 [1] http://www.iphonedevsdk.com/forum/iphone-sdk-development/4717-nsmutablearray-integers.html (accessed 10 January 2010)
 */

#import "World.h"
#import "UnitController.h"
#import "StatObj.h";
#import "DynObjController.h";

@implementation World


- (id) initWorld {
	[super init];
	m_ptGlobalCoordins.x = 0; m_ptGlobalCoordins.y = 0;
	//----------------------------------- rewrite program settings if they exist:
	NSString *filePath = [self dataFilePath];
	//------------- if file exists, copy its content into array and create dynamic objects:
	if ([[NSFileManager defaultManager] fileExistsAtPath:filePath]) {
		NSArray *array = [[NSArray alloc] initWithContentsOfFile:filePath];
		int num = [array count];
		for (int i=0;i<num;i++ ) {
			Byte objType = [[array objectAtIndex:i] intValue];
			switch (objType) {
				case OBJ_UNIT:
					[self AddUnitAt:CGPointMake(0,0) rot:0 ];
					break;
				case OBJ_OPENINGHOLE:
					[self AddDynamicObjectAt:CGPointMake(0,0) rot:0 type:OBJ_OPENINGHOLE];
					break;
			}
		}
		[array release];
	} else {
		//-------------- when application runs for the first time:
		[self AddUnitAt:CGPointMake(-200,-120) rot:0 ];
		[self AddUnitAt:CGPointMake(-100,100) rot:90 ];
		[self AddUnitAt:CGPointMake(100,-150) rot:180 ];
		[self AddUnitAt:CGPointMake(200,50) rot:270 ];
		[self AddDynamicObjectAt:CGPointMake(150,150) rot:0 type:OBJ_OPENINGHOLE];
	}
	[self AddStaticObjectAt:CGPointMake(0,0) rot:0 type:OBJ_BIG];
	
	//--------- get reference to the application instance
	UIApplication *app = [UIApplication sharedApplication];
	
	//---------- subscribe to the notification, using this class as observer (implements a method that handles it specified as selector. Name of the notification is a keyword. Object specifies which object notifies.
	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(applicationWillTerminate:)
												 name:UIApplicationWillTerminateNotification 
											   object:app];
	return self;
	
}

- (void) dealloc {
	for (NSInteger i=0; i< m_iNumStaticObjects; i++) {
		[m_pStaticObjects[i] dealloc];
		[m_pStaticObjects[i] release];
	}
	[super dealloc];
}

- (void) DoFrame {
	//----------------------------------------------------- UPDATE WORLD
	for (NSInteger i=0; i< m_iNumStaticObjects; i++) {
		[m_pStaticObjects[i] Update];
	}
	for (NSInteger i=0; i< m_iNumDynamicObjects; i++) {
		[m_pDynamicObjects[i] Update];
	}
	//----------------------------------------------------- RENDER WORLD
	DrawSpriteAt(TEX_TERRAIN, m_ptGlobalCoordins,0); // draw terrain

	for (NSInteger i=0; i< m_iNumStaticObjects; i++) {
		[m_pStaticObjects[i] Render];
	}
	for (NSInteger i=0; i< m_iNumDynamicObjects; i++) {
		[m_pDynamicObjects[i] Render];
	}
	
	
}


#pragma mark =================================== OBJECTS MANIPULATION

- (void) AddUnitAt: (CGPoint)pos_ rot:(CGFloat)rot_   {
	if (m_iNumDynamicObjects < MAX_DYN_OBJS) {
		m_pDynamicObjects[m_iNumDynamicObjects] = [[UnitController alloc] initUnit:m_iNumDynamicObjects++  pos:pos_ rot:rot_ world:self];
	}
}

/* AddUnitAt: rot: withDNA:
 - used when units breed to create a unit with a specific DNA string
 */
- (void) AddUnitAt: (CGPoint)pos_ rot:(CGFloat)rot_ withDNA:(NSString*)dna_ {
	if (m_iNumDynamicObjects < MAX_DYN_OBJS) {
		m_pDynamicObjects[m_iNumDynamicObjects] = [[UnitController alloc] initUnit:m_iNumDynamicObjects++  pos:pos_ rot:rot_ world:self DNA:dna_];
	}
	
}

- (void) AddDynamicObjectAt: (CGPoint)pos_ rot:(CGFloat)rot_ type:(Byte)type_ {
	m_pDynamicObjects[m_iNumDynamicObjects] = [[DynObjController alloc] initDynObj:m_iNumDynamicObjects++ type:type_ pos:pos_ rot:rot_ world:self];
}

- (void) AddStaticObjectAt: (CGPoint)pos_ rot:(CGFloat)rot_ type:(Byte)type_ {
	m_pStaticObjects[m_iNumStaticObjects] = [[StatObj alloc] initStatObj:m_iNumStaticObjects++ type:type_ pos:pos_ rot:rot_];
}


#pragma mark ======================== TOUCHES

- (void) HandleTouchesBeginAt:(CGPoint)loc_ {
	for (NSInteger i=0; i< m_iNumDynamicObjects; i++) {
		[m_pDynamicObjects[i] HandleTouchesBeginAt:loc_];
	}
}
- (void) HandleTouchesMovedAt:(CGPoint)loc_ {
	for (NSInteger i=0; i< m_iNumDynamicObjects; i++) {
		[m_pDynamicObjects[i] HandleTouchesMovedAt:loc_];
	}
}
- (void) HandleTouchesEndedAt:(CGPoint)loc_ {
	for (NSInteger i=0; i< m_iNumDynamicObjects; i++) {
		[m_pDynamicObjects[i] HandleTouchesEndedAt:(CGPoint)loc_];
	}
}

#pragma mark =========================== OTHER
/* GetDynObjWithId:
 - returns pointer to a dynamic object of a given array id
*/
- (DynObjController*) GetDynObjWithId:(NSInteger)id_ {
	return m_pDynamicObjects[id_];
}

/* GetDynObjAt:
 - returns array id of a dynamic object at a certain position
 - returns -1 if no object found
 */

- (int) GetDynObjIdAt: (CGPoint)pos_ {
	int returnVal = -1;
	for (int i=0; i< m_iNumDynamicObjects; i++) {
		if ([m_pDynamicObjects[i] HitTest:pos_]) {
			returnVal = i;
		}
		
	}
	return returnVal;
}


- (void) UnselectAllOtherObjects:(NSUInteger)id_{
	for (int i=0;i<MAX_DYN_OBJS; i++) {
		if (i != id_) {
		[m_pDynamicObjects[i] Unselect];
		}
		
	}
}


#pragma mark ============================== PERSISTENCE
- (NSString *) dataFilePath {
	//--- look for the Documents directory within the application's sandbox
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
	NSString *documentsDir = [paths objectAtIndex:0];
	//------- append the filename to get its full path:
	return [documentsDir stringByAppendingPathComponent:SETTINGS_FILE_NAME];
}


- (void) applicationWillTerminate: (NSNotification *) notification {
	NSMutableArray *array = [[NSMutableArray alloc] init];
	for (int i=0; i<m_iNumDynamicObjects;i++) {
		[array addObject:[NSNumber numberWithInteger:m_pDynamicObjects[i].m_iType]]; // [REF 1]
	} 
	//-------------- writing atomically -> only rewrite the file if save complete
	[array writeToFile:[self dataFilePath] atomically:YES];
	[array release];
}

@end
